/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2020  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////


#include <QApplication>
#include <QComboBox>
#include <QGridLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QLineEdit>
#include <QListWidget>
#include <QListWidgetItem>
#include <QSizePolicy>
#include <QSpacerItem>
#include <QStringList>
#include <QToolButton>
#include <QFrame>
#include <QVBoxLayout>
#include <QIcon>

#include "labelwidget.h"

static const int						LIST_WIDGET_ITEM_ID		= Qt::UserRole;
static const int						LIST_WIDGET_ITEM_VALUE	= Qt::UserRole + 1;
static const int						LIST_WIDGET_ITEM_TEXT	= Qt::UserRole + 2;
static const int						LIST_WIDGET_ITEM_BORDER	= Qt::UserRole + 3;

TLabelWidget::TLabelWidget(
	const QWidget						*parent,
	const Qt::WindowFlags				flags)
:QWidget(const_cast<QWidget*>(parent),flags)
{
	QGridLayout							*label_option_layout;
	QHBoxLayout							*label_button_hlayout;
	QLabel								*item_value_label;
	QLabel								*item_text_label;
	QLabel								*border_type_label;
	QSizePolicy							size_policy_minimum_fixed(QSizePolicy::Minimum,QSizePolicy::Fixed);
	QSpacerItem							*label_button_hspacer;
	QVBoxLayout							*widget_vlayout;

	size_policy_minimum_fixed.setHorizontalStretch(0);
	size_policy_minimum_fixed.setVerticalStretch(0);

	widget_vlayout = new QVBoxLayout(this);

	d_item_list = new QListWidget(this);
	d_item_list->setSelectionMode(QAbstractItemView::ExtendedSelection);
	d_item_list->setDragDropMode(QAbstractItemView::InternalMove);
	widget_vlayout->addWidget(d_item_list);

	label_button_hlayout = new QHBoxLayout();

	label_button_hspacer = new QSpacerItem(0,0,QSizePolicy::Expanding,QSizePolicy::Minimum);
	label_button_hlayout->addItem(label_button_hspacer);
	
	d_add_item_button = new QToolButton(this);
	d_add_item_button->setMinimumSize(22,22);
	d_add_item_button->setMaximumSize(22,22);
	label_button_hlayout->addWidget(d_add_item_button);

	d_remove_item_button = new QToolButton(this);
	d_remove_item_button->setMinimumSize(22,22);
	d_remove_item_button->setMaximumSize(22,22);
	label_button_hlayout->addWidget(d_remove_item_button);
	widget_vlayout->addLayout(label_button_hlayout);

	label_option_layout = new QGridLayout();

	item_value_label = new QLabel(this);
	label_option_layout->addWidget(item_value_label,0,0,1,1);

	d_item_value_edit = new QLineEdit(this);
	d_item_value_edit->setMaxLength(32);
	d_item_value_edit->setAlignment(Qt::AlignCenter);
	label_option_layout->addWidget(d_item_value_edit,0,1,1,1);
	
	item_text_label = new QLabel(this);
	label_option_layout->addWidget(item_text_label,1,0,1,1);
	
	d_item_text_edit = new QLineEdit(this);
	d_item_text_edit->setMaxLength(32);
	d_item_text_edit->setAlignment(Qt::AlignCenter);
	label_option_layout->addWidget(d_item_text_edit,1,1,1,1);

	border_type_label = new QLabel(this);
	label_option_layout->addWidget(border_type_label,2,0,1,1);

	d_item_border_combo = new QComboBox(this);
	label_option_layout->addWidget(d_item_border_combo,2,1,1,1);
	
	widget_vlayout->addLayout(label_option_layout);
	
	d_add_item_button->setToolTip(QStringLiteral("Add Item"));
	d_remove_item_button->setToolTip(QStringLiteral("Remove Item"));
	d_item_value_edit->setToolTip(QStringLiteral("Item Value"));
	d_item_text_edit->setToolTip(QStringLiteral("Item Text"));
	d_item_border_combo->setToolTip(QStringLiteral("Border Type"));
	
	d_remove_item_button->setText(QStringLiteral("-"));
	d_add_item_button->setText(QStringLiteral("+"));
	item_value_label->setText(QStringLiteral("Item Value:"));
	item_text_label->setText(QStringLiteral("Item Text:"));
	border_type_label->setText(QStringLiteral("Border Type:"));
	
	d_item_border_combo->addItem(QIcon(QStringLiteral(":/gui/outline_none.png")),QStringLiteral("No Border"),TBorderType::BORDER_TYPE_NONE);
	d_item_border_combo->addItem(QIcon(QStringLiteral(":/gui/outline_ellipse.png")),QStringLiteral("Ellipse"),TBorderType::BORDER_TYPE_ELLIPSE);
	d_item_border_combo->addItem(QIcon(QStringLiteral(":/gui/outline_square.png")),QStringLiteral("Square"),TBorderType::BORDER_TYPE_SQUARE);
	d_item_border_combo->addItem(QIcon(QStringLiteral(":/gui/outline_triangle.png")),QStringLiteral("Triangle"),TBorderType::BORDER_TYPE_TRIANGLE);
	d_item_border_combo->addItem(QIcon(QStringLiteral(":/gui/outline_diamond.png")),QStringLiteral("Diamond"),TBorderType::BORDER_TYPE_DIAMOND);
	
	// defaults
	this->Set_Border_Type(TBorderType::BORDER_TYPE_ELLIPSE);
	d_remove_item_button->setEnabled(false);
	d_item_value_edit->setText(QStringLiteral("1"));
	d_current_value = 1;
	d_display_format = QStringLiteral("Item [%1] %2");
	
	connect(d_add_item_button,SIGNAL(clicked(void)),this,SIGNAL(Add_Item(void)));
	connect(d_remove_item_button,SIGNAL(clicked(void)),this,SIGNAL(Remove_Item(void)));
	connect(d_item_list,SIGNAL(itemSelectionChanged(void)),this,SLOT(Item_Selection_Changed(void)));
	
	connect(d_item_value_edit,SIGNAL(textEdited(const QString&)),this,SLOT(Current_Value_Changed(const QString&)));
	connect(d_item_text_edit,SIGNAL(editingFinished(void)),this,SLOT(Current_Text_Changed(void)));
	connect(d_item_border_combo,SIGNAL(currentIndexChanged(int)),this,SLOT(Border_Type_Changed(void)));
}

TLabelWidget::~TLabelWidget(void)
{
}

void TLabelWidget::Item_Selection_Changed(void)
{
	QList<QListWidgetItem*>				sel_items;
	int									sel_items_count;
	int									value;
	int									border_type;
	QString								text;
	bool								prev_state_value;
	bool								prev_state_text;
	bool								prev_state_border;
	
	sel_items = d_item_list->selectedItems();
	sel_items_count = sel_items.size();
	
	d_remove_item_button->setEnabled(sel_items_count);

	prev_state_value = d_item_value_edit->blockSignals(true);
	prev_state_text = d_item_text_edit->blockSignals(true);
	prev_state_border = d_item_border_combo->blockSignals(true);
	
	if(sel_items_count == 0)
	{
		d_item_value_edit->setText(QString("%1").arg(d_current_value));
		d_item_text_edit->setText(QString());
		d_item_text_edit->setEnabled(false);
	}
	else if(sel_items_count == 1)
	{		
		value = sel_items[0]->data(LIST_WIDGET_ITEM_VALUE).toInt();
		d_item_value_edit->setText(QString("%1").arg(value));
		
		text = sel_items[0]->data(LIST_WIDGET_ITEM_TEXT).toString();
		d_item_text_edit->setText(text);
		
		border_type = sel_items[0]->data(LIST_WIDGET_ITEM_BORDER).toInt();
		this->Set_Border_Type(border_type);

		d_item_text_edit->setEnabled(true);
	}
	else
	{
		d_item_value_edit->setText(QString());
		d_item_text_edit->setText(QString());
		d_item_text_edit->setEnabled(false);
	}
	
	d_item_value_edit->blockSignals(prev_state_value);
	d_item_text_edit->blockSignals(prev_state_text);
	d_item_border_combo->blockSignals(prev_state_border);

	emit Selection_Changed();
}

void TLabelWidget::Border_Type_Changed(void)
{
	QList<QListWidgetItem*>				sel_items;
	int									sel_items_count;
	int									id;
	int									cntr;
	int									border_type(TBorderType::BORDER_TYPE_NONE);
	QIcon								icon;
	
	border_type = d_item_border_combo->currentData(Qt::UserRole).toInt();
	
	sel_items = d_item_list->selectedItems();
	sel_items_count = sel_items.size();
	
	if(sel_items_count)
	{
		switch(border_type)
		{
			case TBorderType::BORDER_TYPE_NONE:
			default:
				icon = QIcon(":/gui/outline_none.png");
				break;
				
			case TBorderType::BORDER_TYPE_ELLIPSE:
				icon = QIcon(":/gui/outline_ellipse.png");
				break;
				
			case TBorderType::BORDER_TYPE_SQUARE:
				icon = QIcon(":/gui/outline_square.png");
				break;
				
			case TBorderType::BORDER_TYPE_TRIANGLE:
				icon = QIcon(":/gui/outline_triangle.png");
				break;
				
			case TBorderType::BORDER_TYPE_DIAMOND:
				icon = QIcon(":/gui/outline_diamond.png");
				break;
		}

		for(cntr = 0;cntr < sel_items_count;++cntr)
		{
			sel_items[cntr]->setData(LIST_WIDGET_ITEM_BORDER,border_type);
			sel_items[cntr]->setIcon(icon);
			
			id = sel_items[cntr]->data(LIST_WIDGET_ITEM_ID).toInt();
			emit Change_Item_Border(id,border_type);
		}
	}
}

void TLabelWidget::Current_Value_Changed(
	const QString						&text_value)
{
	QList<QListWidgetItem*>				sel_items;
	QString								text;
	int									sel_items_count;
	int									id;
	int									value;
	bool								state;
	
	if(!text_value.length())
	{
		return;
	}
	
	value = text_value.toInt(&state);
	
	if(!state)
	{
		return;
	}

	sel_items = d_item_list->selectedItems();
	sel_items_count = sel_items.size();

	if(sel_items_count == 0)
	{
		d_current_value = value;
	}
	else if(sel_items_count == 1)
	{
		sel_items[0]->setData(LIST_WIDGET_ITEM_VALUE,value);
		
		text = sel_items[0]->data(LIST_WIDGET_ITEM_TEXT).toString();
		id = sel_items[0]->data(LIST_WIDGET_ITEM_ID).toInt();
		
		if(d_display_format.contains("%1") && d_display_format.contains("%2"))
		{
			sel_items[0]->setText(d_display_format.arg(value).arg(text));
		}
		else if(d_display_format.contains("%1"))
		{
			sel_items[0]->setText(d_display_format.arg(value) + QString(" %2").arg(text));
		}
		else
		{
			sel_items[0]->setText(QString("Item [%1] %2").arg(value).arg(text));
		}

		
		emit Change_Item_Value(id,value);
	}
}

void TLabelWidget::Current_Text_Changed(void)
{
	QList<QListWidgetItem*>				sel_items;
	int									sel_items_count;
	int									id;
	int									value;
	QString								text;
	
	text = d_item_text_edit->text().simplified();
	
	sel_items = d_item_list->selectedItems();
	sel_items_count = sel_items.size();
	
	if(sel_items_count == 1)
	{
		sel_items[0]->setData(LIST_WIDGET_ITEM_TEXT,text);
		
		value = sel_items[0]->data(LIST_WIDGET_ITEM_VALUE).toInt();
		id = sel_items[0]->data(LIST_WIDGET_ITEM_ID).toInt();
		
		if(d_display_format.contains("%1") && d_display_format.contains("%2"))
		{
			sel_items[0]->setText(d_display_format.arg(value).arg(text));
		}
		else if(d_display_format.contains("%1"))
		{
			sel_items[0]->setText(d_display_format.arg(value) + QString(" %2").arg(text));
		}
		else
		{
			sel_items[0]->setText(QString("Item [%1] %2").arg(value).arg(text));
		}
		
		emit Change_Item_Text(id,text);
	}
}

int TLabelWidget::Border_Type(void) const
{
	int									type(TBorderType::BORDER_TYPE_NONE);
	
	if(!(d_item_border_combo->currentIndex() < 0))
	{
		type = d_item_border_combo->currentData().toInt();
	}
	
	return type;
}

std::vector<int> TLabelWidget::Selected_Items(void) const
{
	int									id;
	std::vector<int>					list;
	QList<QListWidgetItem*>				sel_items;
	QList<QListWidgetItem*>::const_iterator iter;
	
	sel_items = d_item_list->selectedItems();
	
	for(iter = sel_items.begin();iter != sel_items.end();++iter)
	{
		id = (*iter)->data(LIST_WIDGET_ITEM_ID).toInt();
		list.push_back(id);
	}
	
	return list;
}

void TLabelWidget::Set_Border_Type(
	const int							border_type)
{
	bool								prev_state;
	
	prev_state = d_item_border_combo->blockSignals(true);

	switch(border_type)
	{
		case TBorderType::BORDER_TYPE_ELLIPSE:
			d_item_border_combo->setCurrentIndex(1);
			break;
			
		case TBorderType::BORDER_TYPE_SQUARE:
			d_item_border_combo->setCurrentIndex(2);
			break;
			
		case TBorderType::BORDER_TYPE_TRIANGLE:
			d_item_border_combo->setCurrentIndex(3);
			break;
			
		case TBorderType::BORDER_TYPE_DIAMOND:
			d_item_border_combo->setCurrentIndex(4);
			break;
			
		case TBorderType::BORDER_TYPE_NONE:
			d_item_border_combo->setCurrentIndex(0);
			break;
	}
	
	d_item_border_combo->blockSignals(prev_state);
}

void TLabelWidget::Set_Label_Index(
	const int							label_index)
{
	d_item_value_edit->setText(QString("%1").arg(label_index));
}

void TLabelWidget::Clear(void)
{
	d_item_list->clear();
}

void TLabelWidget::Add_Item(
	const TDataItem						&data_item)
{
	QListWidgetItem						*list_item;

	list_item = new QListWidgetItem(d_item_list);
	
	if(d_display_format.contains("%1") && d_display_format.contains("%2"))
	{
		list_item->setText(d_display_format.arg(data_item.value).arg(data_item.text));
	}
	else if(d_display_format.contains("%1"))
	{
		list_item->setText(d_display_format.arg(data_item.value) + QString(" %2").arg(data_item.text));
	}
	else
	{
		list_item->setText(QString("Item [%1] %2").arg(data_item.value).arg(data_item.text));
	}
	
	switch(data_item.border_type)
	{
		case TBorderType::BORDER_TYPE_NONE:
		default:
			list_item->setIcon(QIcon(":/gui/outline_none.png"));
			break;
			
		case TBorderType::BORDER_TYPE_ELLIPSE:
			list_item->setIcon(QIcon(":/gui/outline_ellipse.png"));
			break;
			
		case TBorderType::BORDER_TYPE_SQUARE:
			list_item->setIcon(QIcon(":/gui/outline_square.png"));
			break;
			
		case TBorderType::BORDER_TYPE_TRIANGLE:
			list_item->setIcon(QIcon(":/gui/outline_triangle.png"));
			break;
			
		case TBorderType::BORDER_TYPE_DIAMOND:
			list_item->setIcon(QIcon(":/gui/outline_diamond.png"));
			break;
	}
	
	list_item->setData(LIST_WIDGET_ITEM_ID,data_item.id);
	list_item->setData(LIST_WIDGET_ITEM_VALUE,data_item.value);
	list_item->setData(LIST_WIDGET_ITEM_TEXT,data_item.text);
	list_item->setData(LIST_WIDGET_ITEM_BORDER,data_item.border_type);
}

void TLabelWidget::Remove_Item(
	const int							id)
{
	int									count;
	int									cntr;
	QListWidgetItem						*item;
	int									item_id;
	
	count = d_item_list->count();
	
	for(cntr = 0;cntr < count;++cntr)
	{
		item = d_item_list->item(cntr);
		item_id = item->data(LIST_WIDGET_ITEM_ID).toInt();
		
		if(item_id == id)
		{
			d_item_list->takeItem(cntr);
			delete item;
			
			return;
		}
	}
}

void TLabelWidget::Clear_Selection(void)
{
	d_item_list->clearSelection();
}

void TLabelWidget::Set_Selection(
	const int							id)
{
	int									count;
	int									cntr;
	int									item_id;
	QListWidgetItem						*item;
	
	d_item_list->clearSelection();
	
	count = d_item_list->count();
	
	for(cntr = 0;cntr < count;++cntr)
	{
		item = d_item_list->item(cntr);
		item_id = item->data(LIST_WIDGET_ITEM_ID).toInt();
		
		if(item_id == id)
		{
			d_item_list->setCurrentItem(item,QItemSelectionModel::SelectCurrent);
			return;
		}
	}
}

void TLabelWidget::Add_Selection(
	const int							id)
{
	int									count;
	int									cntr;
	int									item_id;
	QListWidgetItem						*item;
		
	count = d_item_list->count();
	
	for(cntr = 0;cntr < count;++cntr)
	{
		item = d_item_list->item(cntr);
		item_id = item->data(LIST_WIDGET_ITEM_ID).toInt();
		
		if(item_id == id)
		{
			d_item_list->setCurrentItem(item,QItemSelectionModel::SelectCurrent);
			return;
		}
	}
}



